@@ -730,6 +730,7 @@ class Payment(CheckoutMixin, metaclass=PoolMeta):
         try:
             return stripe.PaymentIntent.retrieve(
                 self.stripe_payment_intent_id,
+                expand=['latest_charge'],
                 api_key=self.journal.stripe_account.secret_key,
                 stripe_version=STRIPE_VERSION)
         except (stripe.error.RateLimitError,
@@ -742,6 +743,49 @@ class Payment(CheckoutMixin, metaclass=PoolMeta):
     def stripe_intent_update(cls, payments=None):
         pass
 
+    @property
+    def stripe_charge_(self):
+        if not self.stripe_charge_id:
+            return
+        try:
+            return stripe.Charge.retrieve(
+                self.stripe_charge_id,
+                api_key=self.journal.stripe_account.secret_key,
+                stripe_version=STRIPE_VERSION)
+        except (stripe.error.RateLimitError,
+                stripe.error.APIConnectionError) as e:
+            logger.warning(str(e))
+
+    @classmethod
+    def stripe_pull(cls, payments):
+        "Update payments with Stripe charges"
+        for payment in payments:
+            charge = payment.stripe_charge_
+            payment_intent = payment.stripe_payment_intent
+            if payment_intent:
+                charge = payment_intent.latest_charge
+            if charge:
+                payment.stripe_update(charge)
+
+    def stripe_update(self, charge):
+        assert (
+            (charge.id == self.stripe_charge_id)
+            or (charge.payment_intent.id == self.stripe_payment_intent_id))
+        amount = charge.amount - charge.amount_refunded
+        if (self.state not in {'succeeded', 'failed'}
+                or self.stripe_amount != amount
+                or (not amount and self.state != 'failed')):
+            if self.state == 'succeeded':
+                self.__class__.proceed([self])
+            self.stripe_captured = charge.captured
+            self.stripe_amount = amount
+            self.save()
+            if self.amount:
+                if charge.status == 'succeeded' and charge.captured:
+                    self.__class__.succeed([self])
+            else:
+                self.__class__.fail([self])
+
 
 class Refund(Workflow, ModelSQL, ModelView):
     "Stripe Payment Refund"
@@ -814,6 +858,7 @@ class Refund(Workflow, ModelSQL, ModelView):
                 ('approved', 'processing'),
                 ('processing', 'succeeded'),
                 ('processing', 'failed'),
+                ('succeeded', 'failed'),
                 ('approved', 'draft'),
                 ))
         cls._buttons.update({
@@ -1127,8 +1172,10 @@ class Account(ModelSQL, ModelView):
 
     def webhook_charge_refund_updated(self, payload):
         pool = Pool()
+        Payment = pool.get('account.payment')
         Refund = pool.get('account.payment.stripe.refund')
 
+        payments = []
         rf = payload['object']
         refunds = Refund.search([
                 ('stripe_refund_id', '=', rf['id']),
@@ -1143,7 +1190,9 @@ class Account(ModelSQL, ModelView):
             elif rf['status'] in {'failed', 'canceled'}:
                 refund.stripe_error_code = rf['failure_reason']
                 Refund.fail([refund])
+                payments.append(refund.payment)
             refund.save()
+        Payment.stripe_pull(payments)
         return bool(refunds)
 
     def webhook_charge_failed(self, payload, _event='charge.failed'):
@@ -1290,7 +1339,8 @@ class Account(ModelSQL, ModelView):
                 Payment.proceed([payment])
             payment.stripe_captured = bool(
                 payment_intent['amount_received'])
-            payment.stripe_amount = payment_intent['amount_received']
+            if payment.stripe_amount > payment_intent['amount_received']:
+                payment.stripe_amount = payment_intent['amount_received']
             payment.save()
             if payment.amount:
                 Payment.succeed([payment])
